package top.daxian1218.dust.commons.data;

import org.apache.commons.lang3.StringUtils;
import java.math.BigDecimal;

/**
 * 数据列
 *
 * @author hst
 */
public class HDataColumn {
    private String name;
    private String alias;

    private int index = 0;

    private int srcIndex = 0;

    private boolean primaryKey;
    private String dataType = StringUtils.EMPTY;

    private StatType statType = StatType.NONE;

    private BigDecimal statValue;

    private boolean ignore;

    private final ColumnProperties properties = new ColumnProperties();

    public HDataColumn(String name) {
        if (StringUtils.isEmpty(name)) {
            throw new NullPointerException("name");
        }
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public HDataColumn setName(String name) {
        if (StringUtils.isEmpty(name)) {
            throw new NullPointerException("name");
        }
        this.name = name;
        return this;
    }

    /**
     * 如果没有设置，则取name
     * @see HDataColumn#getName()
     * @return
     */
    public String getAlias() {
        return StringUtils.isEmpty(alias) ? this.name : this.alias;
    }

    public HDataColumn setAlias(String alias) {
        this.alias = alias;
        return this;
    }

    public boolean isPrimaryKey() {
        return primaryKey;
    }

    public HDataColumn setPrimaryKey(boolean primaryKey) {
        this.primaryKey = primaryKey;
        return this;
    }

    public ColumnProperties getProperties() {
        return properties;
    }

    public String getDataType() {
        return dataType;
    }

    public HDataColumn setDataType(String dataType) {
        this.dataType = dataType;
        return this;
    }

    public StatType getStatType() {
        return statType;
    }

    public HDataColumn setStatType(StatType statType) {
        this.statType = statType;
        return this;
    }

    public BigDecimal getStatValue() {
        return statValue;
    }

    public HDataColumn setStatValue(BigDecimal statValue) {
        this.statValue = statValue;
        return this;
    }

    public int getIndex() {
        return index;
    }

    public void setIndex(int index) {
        this.index = index;
    }

    public int getSrcIndex() {
        return srcIndex;
    }

    public void setSrcIndex(int srcIndex) {
        this.srcIndex = srcIndex;
    }

    public boolean isIgnore() {
        return ignore;
    }

    public void setIgnore(boolean ignore) {
        this.ignore = ignore;
    }

    @Override
    public String toString() {
        return name + "-" + alias + "-" + dataType;
    }
}


