package top.daxian1218.dust.commons.data;

import java.util.*;
import java.util.stream.Stream;

public class HDataRowCollection implements Iterable<HDataRow> {
    private HDataTable table;
    private final List<HDataRow> data = new ArrayList<>();

    HDataRowCollection(HDataTable table) {
        this.table = table;
    }

    public int size() {
        return data.size();
    }

    public boolean isEmpty() {
        return data.isEmpty();
    }

    public boolean contains(HDataRow o) {
        return data.contains(o);
    }

    @Override
    public Iterator<HDataRow> iterator() {
        return data.iterator();
    }

    public HDataRowCollection add(HDataRow hDataRow) {
        if (hDataRow.getTable() != table) {
            throw new IllegalArgumentException("hDataRow not belong to table");
        }

        if (!contains(hDataRow)) {
            data.add(hDataRow);
            hDataRow.setDetached(false);
        }
        return this;
    }

    public HDataRowCollection remove(HDataRow hDataRow) {
        if (hDataRow.getTable() != table) {
            throw new IllegalArgumentException("hDataRow not belong to table");
        }

        if (data.remove(hDataRow)) {
            hDataRow.setDetached(true);
        }
        return this;
    }

    public void clear() {
        data.forEach(r -> r.setDetached(true));
        data.clear();
    }

    public HDataRow get(int index) {
        return data.get(index);
    }

    public HDataRow set(int index, HDataRow hDataRow) {
        if (hDataRow.getTable() != this.table) {
            throw new IllegalArgumentException("hDataRow not belong to table");
        }

        HDataRow oldRow = data.set(index, hDataRow);
        oldRow.setDetached(true);
        hDataRow.setDetached(false);
        return hDataRow;
    }

    public void add(int index, HDataRow hDataRow) {
        if (hDataRow.getTable() != this.table) {
            throw new IllegalArgumentException("hDataRow not belong to table");
        }
        if (contains(hDataRow)) {
            return;
        }

        data.add(index, hDataRow);
        hDataRow.setDetached(false);
    }

    void moveRow(HDataRow row, int to) {
        if (!contains(row)) {
            throw new IllegalArgumentException("only move row of table");
        }

        data.remove(row);
        data.add(to, row);
    }

    public HDataRow removeAt(int index) {
        if (index < 0 || index >= data.size()) {
            return null;
        }

        HDataRow r = data.get(index);
        remove(r);
        return r;
    }

    public void removeColumnAt(int index) {
        for (HDataRow r : data) {
            r.removeAt(index);
        }
    }


    public int indexOf(HDataRow o) {
        return data.indexOf(o);
    }

    public int lastIndexOf(HDataRow o) {
        return data.lastIndexOf(o);
    }

    public List<HDataRow> subList(int fromIndex, int toIndex) {
        return data.subList(fromIndex, toIndex);
    }

    public Stream<HDataRow> stream() {
        return data.stream();
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        data.clear();
        table = null;
    }

}
