package top.daxian1218.dust.commons.data;

import com.alibaba.fastjson.JSONObject;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Date;
import java.util.Map;
import java.util.function.Consumer;
import java.util.stream.Stream;

public interface IHDataRecord extends Iterable<String> {
    /**
     * 数据长度
     * @return
     */
    int size();

    /**
     * 是否无数据
     * @return
     */
    boolean isEmpty();

    /**
     * 克隆一个新的
     * @return
     */
    IHDataRecord clone();

    /**
     * 合并两个Record
     * @param src
     * @return
     */
    IHDataRecord merge(IHDataRecord src);

    /**
     * 获取字符串值
     * @param key
     * @return
     */
    String getString(String key);

    /**
     * 获取整数值
     * @param key
     * @return
     */
    Integer getInteger(String key);

    /**
     * 获取长整数值
     * @param key
     * @return
     */
    Long getLong(String key);

    /**
     * 获取十进制小数
     * @param key
     * @return
     */
    BigDecimal getDecimal(String key);

    /**
     * 获取布尔值
     * @param key
     * @return
     */
    Boolean getBoolean(String key);

    /**
     * 获取时间日期
     * @param key
     * @return
     */
    Date getDateTime(String key);

    /**
     * 获取日期，时间为00:00::00
     * @param key
     * @return
     */
    Date getDate(String key);

    /**
     * 设置值
     * @param key
     * @param value
     * @return
     */
    IHDataRecord setValue(String key, Object value);

    /**
     * 根据key获取对应的值
     * @param key
     * @return
     */
    Object getValue(String key);

    /**
     * 是否包含key
     * @param key
     * @return
     */
    boolean containsKey(String key);

    /**
     * 包含值
     * @param value
     * @return
     */
    boolean containsValue(Object value);

    /**
     * 遍历所有的key
     * @param action
     */
    void forEachKey(Consumer<String> action);

    /**
     * 获取指定索引index的值，如果超出界限，返回null
     * index与set的前后有关
     * @param index
     * @return
     */
    Object getValue(int index);

    /**
     * 设置指定索引的值，如果超出界限，会报异常
     * @param index
     * @param value
     * @return
     */
    IHDataRecord setValue(int index, Object value);

    IHDataRecord clear();

    IHDataRecord remove(String key);

    /**
     * 获取key的Stream
     * @return
     */
    Stream<String> stream();

    /**
     * 加载map数据到Record
     * @param map
     * @return
     */
    IHDataRecord load(Map<String, Object> map);

    JSONObject toJSON();

    JSONObject toJSON(boolean camelName);

    Map<String, Object> toMap();

    Collection<Object> values();
}
