package top.daxian1218.dust.commons.thread;


import top.daxian1218.dust.commons.generic.Preconditions;

import java.util.HashMap;
import java.util.Map;

/**
 * 统一管理本地线程内容
 * @author hst on 2020-8-1.
 */
public class LocalHolder {
    private final static ThreadLocal<Map<String, Object>> threadLocal = new ThreadLocal<>();

    public static <T> T get(String key) {
        Preconditions.checkNotNull(key);
        Map<String,Object> map = threadLocal.get();
        if (map != null && map.containsKey(key)) {
            return (T)map.get(key);
        }

        return null;
    }

    public static <T> T getOrDefault(String key, T defaultValue) {
        Preconditions.checkNotNull(key);
        Map<String,Object> map = threadLocal.get();
        if (map != null && map.containsKey(key)) {
            return (T)map.get(key);
        }

        set(key, defaultValue);
        return defaultValue;
    }

    public static void set(String key, Object value) {
        Preconditions.checkNotNull(key);
        Map<String,Object> map = threadLocal.get();
        if (map == null) {
            map = new HashMap<>(0);
            threadLocal.set(map);
        }

        map.put(key, value);
    }

    public static void remove() {
        threadLocal.remove();
    }
}
