package top.daxian1218.dust.commons.util;


import top.daxian1218.dust.commons.log.LoggerHelper;
import top.daxian1218.dust.commons.reflection.IBeanFactory;

/**
 * 用于获取当前SpringContext里面的Bean对象
 *
 * @author hst
 */
public class BeanUtil {

    public static IBeanFactory FACTORY;

    public static Object getBean(String name) {
        if (FACTORY != null) {
            Object bean = FACTORY.getBean(name);
            if (bean != null) { return bean; }

            String ignoreFirstCharName =  NameUtil.toggleCaseFirstLetter(name);
            bean = FACTORY.getBean(ignoreFirstCharName);
            if (bean != null) { return bean; }
        }
        return getBeanByClass(name);
    }

    public static <T> T getBean(Class<T> cls) {
        if (FACTORY != null)
        {
            try {
                return FACTORY.getBean(cls);
            } catch (Exception ex) {
                LoggerHelper.debug(ex);
            }
        }

        return null;
    }

    public static Object getBeanByClass(String clazz) {
        Object beanObj = null;
        try {
            beanObj = Class.forName(clazz).newInstance();
        } catch (Exception ex) {
            LoggerHelper.debug(ex);
        }
        return beanObj;
    }

    public static <T> Iterable<T> getBeans(Class<T> cls) {
        if (FACTORY != null) {
            try {
                return FACTORY.getBeans(cls);
            } catch (Exception ex) {
                LoggerHelper.debug(ex);
            }
        }

        return null;
    }
}
