package top.daxian1218.dust.commons.util;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.data.HDataRow;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.JarURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.util.*;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

/**
 * 对象构建类
 *
 * @author hst
 */
public class ClassUtil {

    public static Object copyClass(Object src) {
        Class cls = src.getClass();
        Object obj = null;
        try {
            obj = cls.newInstance();
        } catch (Exception e) {
            e.printStackTrace();
            return obj;
        }
        Field[] fields = cls.getDeclaredFields();
        for (int i = 0; i < fields.length; i++) {
            Field field = fields[i];
            field.setAccessible(true);
            try {
                field.set(obj, field.get(src));
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }
        return obj;
    }

    public static <T> T mapToObject(HDataRow params, Class<T> type) {
        return mapToObject(params.toMap(), type);
    }

    public static <T> T mapToObject(HDataRow params, T obj, Class<T> type) {
        return mapToObject(params.toMap(), obj, type);
    }

    public static <T> T mapToObject(Map<String, Object> params, Class<T> type) {
        T obj = null;
        try {
            obj = type.newInstance();
        } catch (InstantiationException | IllegalAccessException e) {
            e.printStackTrace();
            return null;
        }

        return mapToObject(params, obj, type);
    }

    /**
     * 按照类T开放的set方法，进行赋值
     * 将map的key不区分大小写，不区分下划线，且支持子对象的赋值，暂未支持子属性是List的情况
     *
     * @param params 数据参数
     * @param obj    目标对象
     * @param type   类型
     * @return
     */
    public static <T> T mapToObject(Map<String, Object> params, T obj, Class<T> type) {

        //获取所有的set方法
        Method[] m = type.getMethods();
        Map<String, Method> setMethods = new HashMap<>();
        for (Method temp : m) {
            if (temp.getName().length() > 3 && temp.getName().startsWith("set")) {
                String fieldName = temp.getName().substring(3);
                setMethods.put(NameUtil.camel2Underscore(fieldName), temp);

                if (fieldName.indexOf("_") < 0) {
                    setMethods.put(NameUtil.toLowerCaseFirstLetter(fieldName), temp);
                }

                // 补充支持大写字符的匹配
                setMethods.put(fieldName.toUpperCase(), temp);
                setMethods.put(NameUtil.camel2Underscore(fieldName).toUpperCase(), temp);
            }
        }

        for (String key : params.keySet()) {
            int doIndex = key.indexOf(".");
            if (doIndex < 0 && setMethods.containsKey(key)) {
                try {
                    setMethods.get(key).invoke(obj, typeConvert(params.get(key), setMethods.get(key).getParameterTypes()[0]));
                } catch (IllegalAccessException e) {

                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                }
            } else if (doIndex >= 0 && setMethods.containsKey(key.substring(0, doIndex))) {
                setSubValue(obj, key, params.get(key));
            }
        }

        return obj;
    }

    private static Object setSubValue(Object obj, String path, Object value) {
        try {
            int index = path.indexOf(".");
            if (index < 0) {
                Method setMethod = findMethod(obj.getClass(), "set" + path, true);
                if (setMethod != null) {
                    setMethod.invoke(obj, value);
                }
            } else {
                String fieldName = path.substring(0, index);
                Method getMethod = findMethod(obj.getClass(), "get" + fieldName);
                Object o = null;
                if (getMethod != null) {
                    o = getMethod.invoke(obj);
                }

                if (o == null) {
                    Method setMethod = findMethod(obj.getClass(), "set" + fieldName, true);
                    if (setMethod != null) {
                        Object newObj = typeConvert(value, setMethod.getParameterTypes()[0]);
                        setSubValue(newObj, path.substring(index + 1), value);
                        setMethod.invoke(obj, newObj);
                    }
                } else {
                    Object newObj = o;
                    setSubValue(newObj, path.substring(index + 1), value);
                }

            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
        return obj;
    }


    private static Method findMethod(Class clz, String lowerName, boolean ignoreObjectMethod) {
        for (Method m : clz.getMethods()) {
            if (StringUtils.equalsIgnoreCase(lowerName, m.getName())) {
                if (ignoreObjectMethod) {
                    Class<?>[] pts = m.getParameterTypes();
                    for (Class c : pts) {
                        if (!c.getSimpleName().equals("Object")) {
                            return m;
                        }
                    }
                } else {
                    return m;
                }
            }
        }

        return null;
    }

    private static Method findMethod(Class clz, String lowerName) {
        return findMethod(clz, lowerName, false);
    }

    public static Object typeConvert(Object obj, Class clazz) {

        if (clazz.isEnum()) {
            return TypeUtil.toEnumByValue(clazz, obj);
        }

        switch (clazz.getSimpleName()) {
            case "Integer":
            case "int":
                return TypeUtil.toInteger(obj);
            case "String":
                return TypeUtil.toString(obj);

            case "Boolean":
            case "boolean":
                return TypeUtil.toBoolean(obj);

            case "Long":
            case "long":
                return TypeUtil.toLong(obj);

            case "Short":
            case "short":
                return new Integer(TypeUtil.toInteger(obj)).shortValue();

            case "Double":
            case "double":
                return TypeUtil.toDouble(obj);

            case "BigDecimal":
                return TypeUtil.toDecimal(obj);

            case "Float":
            case "float":
                return new Double(TypeUtil.toDouble(obj)).floatValue();
            case "Date":
                return TypeUtil.toDate(obj);
            case "Object":
                return obj;
            default:
                try {
                    return clazz.newInstance();
                } catch (Exception ex) {
                    return null;
                }
        }
    }

    /**
     * 取得某个接口下所有实现这个接口的类
     */
    public static List<Class> getAllClassByInterface(Class c) {
        List<Class> returnClassList = null;

        if (c.isInterface()) {
            // 获取当前的包名
            String packageName = c.getPackage().getName();
            // 获取当前包下以及子包下所以的类
            List<Class<?>> allClass = getClasses(packageName);
            if (allClass != null) {
                returnClassList = new ArrayList<>();
                for (Class classes : allClass) {
                    // 判断是否是同一个接口
                    if (c.isAssignableFrom(classes)) {
                        // 本身不加入进去
                        if (!c.equals(classes)) {
                            returnClassList.add(classes);
                        }
                    }
                }
            }
        }

        return returnClassList;
    }


    /**
     * 取得某一类所在包的所有类名 不含迭代
     */
    public static String[] getPackageAllClassName(String classLocation, String packageName) {
        //将packageName分解
        String[] packagePathSplit = packageName.split("[.]");
        String realClassLocation = classLocation;
        int packageLength = packagePathSplit.length;
        for (int i = 0; i < packageLength; i++) {
            realClassLocation = realClassLocation + File.separator + packagePathSplit[i];
        }
        File packeageDir = new File(realClassLocation);
        if (packeageDir.isDirectory()) {
            String[] allClassName = packeageDir.list();
            return allClassName;
        }
        return null;
    }

    /**
     * 从包package中获取所有的Class
     *
     * @param packageName
     * @return
     */
    public static List<Class<?>> getClasses(String packageName) {

        //第一个class类的集合
        List<Class<?>> classes = new ArrayList<Class<?>>();
        //是否循环迭代
        boolean recursive = true;
        //获取包的名字 并进行替换
        String packageDirName = packageName.replace('.', '/');
        //定义一个枚举的集合 并进行循环来处理这个目录下的things
        Enumeration<URL> dirs;
        try {
            dirs = Thread.currentThread().getContextClassLoader().getResources(packageDirName);
            //循环迭代下去
            while (dirs.hasMoreElements()) {
                //获取下一个元素
                URL url = dirs.nextElement();
                //得到协议的名称
                String protocol = url.getProtocol();
                //如果是以文件的形式保存在服务器上
                if ("file".equals(protocol)) {
                    //获取包的物理路径
                    String filePath = URLDecoder.decode(url.getFile(), "UTF-8");
                    //以文件的方式扫描整个包下的文件 并添加到集合中
                    findAndAddClassesInPackageByFile(packageName, filePath, recursive, classes);
                } else if ("jar".equals(protocol)) {
                    //如果是jar包文件
                    //定义一个JarFile
                    JarFile jar;
                    try {
                        //获取jar
                        jar = ((JarURLConnection) url.openConnection()).getJarFile();
                        //从此jar包 得到一个枚举类
                        Enumeration<JarEntry> entries = jar.entries();
                        //同样的进行循环迭代
                        while (entries.hasMoreElements()) {
                            //获取jar里的一个实体 可以是目录 和一些jar包里的其他文件 如META-INF等文件
                            JarEntry entry = entries.nextElement();
                            String name = entry.getName();
                            //如果是以/开头的
                            if (name.charAt(0) == '/') {
                                //获取后面的字符串
                                name = name.substring(1);
                            }
                            //如果前半部分和定义的包名相同
                            if (name.startsWith(packageDirName)) {
                                int idx = name.lastIndexOf('/');
                                //如果以"/"结尾 是一个包
                                if (idx != -1) {
                                    //获取包名 把"/"替换成"."
                                    packageName = name.substring(0, idx).replace('/', '.');
                                }
                                //如果可以迭代下去 并且是一个包
                                if ((idx != -1) || recursive) {
                                    //如果是一个.class文件 而且不是目录
                                    if (name.endsWith(".class") && !entry.isDirectory()) {
                                        //去掉后面的".class" 获取真正的类名
                                        String className = name.substring(packageName.length() + 1, name.length() - 6);
                                        try {
                                            //添加到classes
                                            classes.add(Class.forName(packageName + '.' + className));
                                        } catch (ClassNotFoundException e) {
                                            e.printStackTrace();
                                        }
                                    }
                                }
                            }
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

        return classes;
    }

    /**
     * 以文件的形式来获取包下的所有Class
     *
     * @param packageName
     * @param packagePath
     * @param recursive
     * @param classes
     */
    public static void findAndAddClassesInPackageByFile(String packageName, String packagePath, final boolean recursive, List<Class<?>> classes) {
        //获取此包的目录 建立一个File
        File dir = new File(packagePath);
        //如果不存在或者 也不是目录就直接返回
        if (!dir.exists() || !dir.isDirectory()) {
            return;
        }
        //如果存在 就获取包下的所有文件 包括目录
        File[] dirfiles = dir.listFiles(new FileFilter() {
            //自定义过滤规则 如果可以循环(包含子目录) 或则是以.class结尾的文件(编译好的java类文件)
            public boolean accept(File file) {
                return (recursive && file.isDirectory()) || (file.getName().endsWith(".class"));
            }
        });
        //循环所有文件
        for (File file : dirfiles) {
            //如果是目录 则继续扫描
            if (file.isDirectory()) {
                findAndAddClassesInPackageByFile(packageName + "." + file.getName(),
                        file.getAbsolutePath(),
                        recursive,
                        classes);
            } else {
                //如果是java类文件 去掉后面的.class 只留下类名
                String className = file.getName().substring(0, file.getName().length() - 6);
                try {
                    //添加到集合中去
                    classes.add(Class.forName(packageName + '.' + className));
                } catch (ClassNotFoundException e) {
                    e.printStackTrace();
                }
            }
        }
    }

}


