package top.daxian1218.dust.commons.util;

/**
 * 命名转化类。驼峰，下划线，大小写命名互转
 *
 * @author hst
 */
public class NameUtil {

    /**
     * 驼峰转下划线
     *
     * @return
     */
    public static String camel2Underscore(String camelName) {
        //先把第一个字母大写
        camelName = capitalize(camelName);

        String regex = "([A-Z][a-z]+)";
        String replacement = "$1_";

        String underscoreName = camelName.replaceAll(regex, replacement);
        //output: Pur_Order_Id_ 接下来把最后一个_去掉，然后全部改小写

        underscoreName = underscoreName.toLowerCase().substring(0, underscoreName.length() - 1);

        return underscoreName;
    }

    /**
     * 驼峰转大写下划线
     *
     * @return
     */
    public static String camel2UpperUnderscore(String camelName) {
        //先把第一个字母大写
        camelName = capitalize(camelName);

        String regex = "([A-Z][a-z]+)";
        String replacement = "$1_";

        String underscoreName = camelName.replaceAll(regex, replacement);

        //output: Pur_Order_Id_ 接下来把最后一个_去掉，然后全部改大写
        underscoreName = underscoreName.toUpperCase().substring(0, underscoreName.length() - 1);

        return underscoreName;
    }

    /**
     * convert underscore name to camel name
     *
     * @param underscoreName
     * @return
     */
    public static String underscore2Camel(String underscoreName) {
        String[] sections = underscoreName.toLowerCase().split("_");
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < sections.length; i++) {
            String s = sections[i];
            if (i == 0) {
                sb.append(s);
            } else {
                sb.append(capitalize(s));
            }
        }
        return sb.toString();
    }

    /**
     * capitalize the first character
     *
     * @param str
     * @return
     */
    public static String capitalize(String str) {
        if (str == null || str.length() == 0) {
            return str;
        }
        if (str.length() == 1) {
            return str.toLowerCase();
        }
        return String.valueOf(Character.toTitleCase(str.charAt(0))) +
                str.substring(1);
    }


    /**
     * 首字母转小写
     *
     * @param s
     * @return
     */
    public static String toLowerCaseFirstLetter(String s) {
        if (Character.isLowerCase(s.charAt(0))) {
            return s;
        }

        return (new StringBuilder()).append(Character.toLowerCase(s.charAt(0))).append(s.substring(1)).toString();
    }

    /**
     * 首字母转大写
     *
     * @param s
     * @return
     */
    public static String toUpperCaseFirstLetter(String s) {
        if (Character.isUpperCase(s.charAt(0))) {
            return s;
        }

        return (new StringBuilder()).append(Character.toUpperCase(s.charAt(0))).append(s.substring(1)).toString();
    }


    /**
     * 转换首字母大小写
     *
     * @param s
     * @return
     */
    public static String toggleCaseFirstLetter(String s) {
        if (Character.isUpperCase(s.charAt(0))) {
            return (new StringBuilder()).append(Character.toLowerCase(s.charAt(0))).append(s.substring(1)).toString();
        }

        return (new StringBuilder()).append(Character.toUpperCase(s.charAt(0))).append(s.substring(1)).toString();
    }
}