package top.daxian1218.dust.commons.util;

import com.alibaba.fastjson.JSON;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import top.daxian1218.dust.commons.log.LoggerHelper;

import java.io.*;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.EnumSet;

/**
 * @author hst
 */
public class TypeUtil {

    public static final String DATETIME_FORMAT = "yyyy-MM-dd HH:mm:ss";
    public static final String DATE_FORMAT = "yyyy-MM-dd";
    public static final String TIME_FORMAT = "HH:mm:ss";
    public static final String EMPTY_STRING = "";

    public static boolean toBoolean(Object value) {
        if (value instanceof Boolean) {
            return (Boolean) value;
        }

        if (value != null) {
            String strValue = toString(value);
            if (strValue.length() == 0) {
                return false;
            }

            char firstChar = strValue.charAt(0);
            return firstChar == '1' || firstChar == 't' || firstChar == 'T';
        }

        return false;
    }


    public static Date toDate(Object value, String format) {
        if (value == null) {
            return null;
        }
        if (value instanceof Date) {
            return (Date) value;
        }

        if (value instanceof Integer) {
            return new Date(((Integer) value).longValue());
        }

        if (value instanceof Short) {
            return new Date(((Short) value).longValue());
        }

        if (value instanceof Long) {
            return new Date((Long) value);
        }

        if (value instanceof String && StringUtils.isNumeric((String) value)) {
            return new Date(toLong(value));
        }

        // 字符串格式的时间日期
        String strDate = toString(value);
        format = null == format ? "yyyy-MM-dd HH:mm:ss" : format;
        if (format.length() < strDate.length() && format.equals(DATE_FORMAT)) {
            format = DATETIME_FORMAT;
        }

        DateFormat dateFormat = new SimpleDateFormat(format);
        try {
            return dateFormat.parse(strDate);
        } catch (ParseException ignored) {
        }
        return null;
    }

    public static Date toDateTime(Object value) {
        return toDate(value, DATETIME_FORMAT);
    }

    public static java.sql.Date toSqlDataTime(String value) {
        Date date = toDateTime(value);
        if (date == null) {
            return null;
        }
        return new java.sql.Date(date.getTime());
    }

    public static Date toDate(Object value) {
        return toDate(value, DATE_FORMAT);
    }

    public static double toDouble(Object value) {
        if (value == null) {
            return 0.0;
        }

        if (value instanceof Double) {
            return (Double) value;
        }

        if (value instanceof BigDecimal) {
            return ((BigDecimal) value).doubleValue();
        }

        try {
            return toDecimal(value.toString()).doubleValue();
        } catch (Exception ignored) {
        }
        return 0.0;
    }

    public static float toFloat(Object value) {
        if (value == null) {
            return 0.0f;
        }

        if (value instanceof Float) {
            return (Float) value;
        }

        if (value instanceof BigDecimal) {
            return ((BigDecimal) value).floatValue();
        }

        try {
            return toDecimal(value.toString()).floatValue();
        } catch (Exception ignored) {
        }
        return 0.0f;
    }

    public static BigDecimal toDecimal(Object value) {
        if (value == null) {
            return new BigDecimal(0);
        }

        if (value instanceof BigDecimal) {
            return (BigDecimal) value;
        }

        BigDecimal bd = new BigDecimal(0);
        try {

            String str = value.toString();
            if (NumberUtils.isParsable(str)) {
                return new BigDecimal(str);
            }
        } catch (Exception ignored) {
        }
        return bd;
    }

    public static int toInteger(Object value) {
        if (value == null) {
            return 0;
        }

        if (value instanceof Integer) {
            return (Integer) value;
        }

        if (value instanceof BigDecimal) {
            return ((BigDecimal) value).intValue();
        }

        try {
            BigDecimal i = toDecimal(value.toString());
            return i.intValue();
        } catch (Exception ignored) {
        }
        return 0;
    }

    public static long toLong(Object value) {
        if (value == null) {
            return 0L;
        }

        if (value instanceof BigDecimal) {
            return ((BigDecimal) value).longValue();
        }

        if (value instanceof Long) {
            return (Long) value;
        }

        try {
            BigDecimal i = toDecimal(value.toString());
            return i.longValue();
        } catch (Exception ignored) {
        }
        return 0L;
    }


    public static String toString(Object o, String... formats) {
        if (o == null) {
            return EMPTY_STRING;
        }

        if (o instanceof Date) {
            String format = formats == null || formats.length == 0 ? null : formats[0];
            return toStringOfDateTime((Date) o, format);
        }

        if (o instanceof Boolean) {
            return (Boolean) o ? "1" : "0";
        }

        //TODO 支持不同类型的转化
        if (o instanceof byte[]) {
            return toHexString((byte[]) o);
        }

        return o.toString();
    }

    public static String toStringOfDate(Date d, String format) {
        if (d == null) {
            return EMPTY_STRING;
        }

        if (StringUtils.isEmpty(format)) {
            format = DATE_FORMAT;
        }

        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(d);
    }

    public static String toStringOfDate(Date d) {
        return toStringOfDate(d, null);
    }

    public static String toStringOfDateTime(Date d, String format) {
        if (d == null) {
            return EMPTY_STRING;
        }

        if (StringUtils.isEmpty(format)) {
            format = DATETIME_FORMAT;
        }

        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(d);
    }

    public static String toStringOfDateTime(Date d) {
        return toStringOfDateTime(d, null);
    }

    public static Object toStringBuffer(Object value) {
        if (value == null) {
            return new StringBuffer(EMPTY_STRING);
        }

        return new StringBuffer(toString(value));
    }

    public static byte[] toBytes(Object value) {
        if (value == null) {
            return null;
        }

        try {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            ObjectOutputStream oos = new ObjectOutputStream(bos);
            oos.writeObject(value);
            oos.flush();
            byte[] bytes = bos.toByteArray();
            oos.close();
            bos.close();
            return bytes;
        } catch (IOException ignored) {
        }
        return null;
    }

    public static Object toObject(byte[] bs) {
        if (bs == null) {
            throw new NullPointerException("bs");
        }

        Object o = null;
        try {
            ByteArrayInputStream bos = new ByteArrayInputStream(bs);
            ObjectInputStream oos = new ObjectInputStream(bos);
            o = oos.readObject();
            oos.close();
            bos.close();
        } catch (Exception ignored) {
        }
        return o;
    }

    public static String toStringEmpty(Object o) {
        if (o == null) {
            return EMPTY_STRING;
        }
        return o.toString();
    }


    /**
     * 根据序号转换为枚举
     *
     * @param clazz
     * @param index
     * @param <T>
     * @return
     */
    public static <T extends Enum<T>> T toEnumByIndex(Class<T> clazz, int index) {
        T[] c = clazz.getEnumConstants();
        return c[index];
    }

    /**
     * 根据枚举名称转换枚举
     *
     * @param clazz
     * @param name
     * @param <T>
     * @return
     */
    public static <T extends Enum<T>> T toEnumByName(Class<T> clazz, Object name) {
        if (null != name) {
            return Enum.valueOf(clazz, name.toString());
        } else {
            return null;
        }
    }

    /**
     * 根据枚举值转换枚举
     *
     * @param clazz
     * @param value
     * @param <T>
     * @return
     */
    public static <T extends Enum<T>> T toEnumByValue(Class<T> clazz, Object value) {
        if (null != value) {
            EnumSet<T> currEnumSet = EnumSet.allOf(clazz);
            for (T dataType : currEnumSet) {
                if (dataType.toString().equals(value.toString())) {
                    return dataType;
                }
            }
        }
        return null;
    }

    public static String toHexStringOfBigInt(byte[] bs) {
        if (bs == null || bs.length == 0) {
            return StringUtils.EMPTY;
        }

        BigInteger bigInt = new BigInteger(1, bs);
        return bigInt.toString(16);
    }

    public static String toHexString(byte[] byteArray) {
        if (byteArray == null || byteArray.length < 1) return StringUtils.EMPTY;


        final StringBuilder hexString = new StringBuilder();
        for (byte b : byteArray) {
            //0~F前面补零
            if ((b & 0xff) < 0x10) {
                hexString.append("0");
            }
            hexString.append(Integer.toHexString(0xFF & b));
        }
        return hexString.toString().toLowerCase();
    }

    public static String toHexString(int[] byteArray) {
        if (byteArray == null || byteArray.length < 1) return StringUtils.EMPTY;


        final StringBuilder hexString = new StringBuilder();
        for (int b : byteArray) {
            //0~F前面补零
            if ((b & 0xff) < 0x10) {
                hexString.append("0");
            }
            hexString.append(Integer.toHexString(0xFF & b));
        }
        return hexString.toString().toLowerCase();
    }

    public static String toStringOrJson(Object value) {
        if (value == null) {
            return StringUtils.EMPTY;
        }

        if (value instanceof String) {
            return (String) value;
        }

        if (value instanceof Date) {
            return toString(TimestampUtil.getTimestamp((Date)value));
        }

        if (value.getClass().isPrimitive()) {
            return toString(value);
        }

        return JSON.toJSONString(value);
    }
}
